"""
Output generators for scheduling models.

This module provides:
- XCSP3 scheduling extension output generator
"""

from __future__ import annotations

# To be implemented:
# from pycsp3_scheduling.output.xcsp3_scheduling import generate_xcsp3_scheduling

__all__: list[str] = []
